--_________________________________________________________________________________________________
--                                                                                                |
--                                       |masterFIP core|                                         |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                         incr_counter                                           |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         incr_counter.vhd                                                                  |
-- Description  Increasing counter with synchronous reinitialise and increase enable              |
-- Authors      Evangelia Gousiou     (Evangelia.Gousiou@cern.ch)                                 |
--                                                                                                |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------



--=================================================================================================
--                                      Libraries & Packages
--=================================================================================================

-- Standard library
library IEEE;
use IEEE.STD_LOGIC_1164.all; -- std_logic definitions
use IEEE.NUMERIC_STD.all;    -- conversion functions

--=================================================================================================
--                           Entity declaration for incr_counter
--=================================================================================================

entity incr_counter is
  generic(g_counter_lgth : natural := 32); -- default length
  port(
  -- INPUTS
   clk_i             : in std_logic;       -- 40 MHz clock
   counter_incr_i    : in std_logic;       -- increment enable
   counter_reinit_i  : in std_logic;       -- reinitializes counter to 0

  -- OUTPUT
   counter_o         : out std_logic_vector (g_counter_lgth-1 downto 0); -- counter
   counter_is_full_o : out std_logic);     -- counter full indication, when all bits are '1'
end entity incr_counter;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture rtl of incr_counter is

constant c_COUNTER_FULL : unsigned (g_counter_lgth-1 downto 0) := (others => '1');
signal   s_counter      : unsigned (g_counter_lgth-1 downto 0);


--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin


---------------------------------------------------------------------------------------------------
  -- Synchronous process Incr_Counter
  Incr_Counter: process (clk_i)
  begin
    if rising_edge (clk_i) then
      if counter_reinit_i = '1' then
        s_counter   <= (others => '0');

      elsif counter_incr_i = '1' then
        s_counter   <= s_counter + 1;

      end if;
    end if;
  end process;

 --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --
  counter_o         <= std_logic_vector(s_counter);
  counter_is_full_o <= '1' when s_counter = c_COUNTER_FULL else '0';


end architecture rtl;
--=================================================================================================
--                                        architecture end
--=================================================================================================
---------------------------------------------------------------------------------------------------
--                                      E N D   O F   F I L E
---------------------------------------------------------------------------------------------------